import { AfterContentChecked, ChangeDetectorRef, Component, OnInit } from '@angular/core';
import { FormGroup, UntypedFormGroup } from '@angular/forms';
import { User } from '../../models/user.model';
import { ProfileService } from '../../profile/services/profile.service';
import { AuthService } from '../../services/auth.service';
import { NotificationService } from '../../services/notification.service';
import { OnlineMediationService } from '../../services/online-mediation.service';
import { Filter } from '../../structure-list/models/filter.model';
import { Module } from '../../structure-list/models/module.model';
import { OrientationUtils } from '../../utils/orientationUtils';
import { Utils } from '../../utils/utils';
import { FiltersSteps } from './enums/filtersSteps.enum';
import { GenericOrientationSteps } from './enums/genericOrientationSteps.enum';
import { HotlineMediationSteps } from './enums/hotlineMediationSteps.enum';
import { NeedsType } from './enums/needs.enum';
import { OnlineDemarche } from './enums/onlineDemarche.enum';
import { OnlineDemarchesCommonSteps } from './enums/onlineDemarchesCommonSteps.enum';
import { OnlineDemarchesMeetingSteps } from './enums/onlineDemarchesMeetingSteps.enum';
import { StructuresListSteps } from './enums/structuresListSteps.enum';
import { FiltersForm } from './interfaces/filtersForm.interface';
import { IOnlineMediation } from './interfaces/onlineMediation.interface';
import { StructureOrientationForm } from './interfaces/structureOrientationForm.interface';
import { MediationType } from './types/mediation.type';
import { MediationStepType } from './types/mediationStep.type';

@Component({
  selector: 'app-orientation-form-view',
  templateUrl: './orientation-form-view.component.html',
  styleUrls: ['./orientation-form-view.component.scss'],
})
export class OrientationFormViewComponent implements OnInit, AfterContentChecked {
  public orientationUtils = new OrientationUtils();
  public utils = new Utils();
  // OnlineDemarches
  public onlineDemarchForm: UntypedFormGroup;
  public onlineDemarcheType: OnlineDemarche;
  // Equipments
  public filtersForm: FormGroup<FiltersForm> | FormGroup<StructureOrientationForm>;
  // Enum
  public needEnum = NeedsType;
  // Global
  public needType: NeedsType;
  public currentType: MediationType = OnlineDemarche.common;
  public currentStep: MediationStepType = null;
  public isLastStep = false;
  public nbSteps: number = Object.keys(HotlineMediationSteps).length / 2;
  public isPageValid: boolean = false;
  public filters: Filter[] = [];
  public profile: User;
  public canDeactivate: boolean = false;
  public showLoginModal: boolean = false;
  public fullScreen: boolean = false;

  // Modal canExit var
  public showConfirmationModal = false;
  private resolve: Function;

  constructor(
    private onlineMediationService: OnlineMediationService,
    private notificationService: NotificationService,
    private authService: AuthService,
    private profileService: ProfileService,
    private cdref: ChangeDetectorRef
  ) {}

  async ngOnInit() {
    if (!this.authService.isLoggedIn()) {
      this.showLoginModal = true;
    } else {
      this.profile = await this.profileService.getProfile();
    }
  }

  ngAfterContentChecked() {
    this.cdref.detectChanges();
  }

  public validatePage(event: boolean) {
    this.isPageValid = event;
  }

  public setStepNumber(enumLength: number): void {
    this.nbSteps = enumLength / 2;
  }

  public setCurrentNeedType(needType: NeedsType): void {
    this.needType = needType;
    this.validatePage(true);
    // Init form according to needType
    switch (needType) {
      case NeedsType.onlineDemarch:
        this.onlineDemarchForm = this.orientationUtils.createOnlineDemarchesForm();
        break;
      case NeedsType.equipmentAccess:
      case NeedsType.equipmentBuy:
        this.filtersForm = this.orientationUtils.createFiltersForm();
        break;
      case NeedsType.learnSkills:
        this.filtersForm = this.orientationUtils.createFiltersForm();
        this.onlineDemarchForm = this.orientationUtils.createOnlineDemarchesForm();
        break;
      default:
        throw new Error(`Not implemented NeedsType in OrientationFormViewComponent ${needType}`);
    }
  }

  public checkLastStep(): void {
    if (
      this.needType === NeedsType.onlineDemarch &&
      this.currentType === OnlineDemarche.onlineMediation &&
      this.currentStep === HotlineMediationSteps.mediationRecap
    ) {
      this.isLastStep = true;
    } else if (this.isStructureListForm() && this.currentStep === StructuresListSteps.mediationRecap) {
      this.isLastStep = true;
    } else this.isLastStep = false;
  }

  /**
   * Handle online mediation finsh by sending appointement to server
   */
  public handleOnlineAppointment(): void {
    const toCreate: IOnlineMediation = {
      ...this.onlineDemarchForm.value,
      onlineDemarchType: this.onlineDemarchForm.value.filters.map((module: Module) => module.displayText),
    };
    this.onlineMediationService.createOnlineMediation(toCreate).subscribe((data) => {
      if (data) {
        this.canDeactivate = true;
        this.notificationService.showSuccess('Votre démarche en ligne a bien été enregistrée');
      } else {
        this.notificationService.showError('Echec de la création de votre démarche en ligne');
      }
      this.printForm();
    });
  }

  public printForm(): void {
    window.print();
    setTimeout(() => {
      this.canDeactivate = true;
      window.location.reload();
    }, 100);
  }

  public setOnlineEquipmentsForm() {
    this.currentType = OnlineDemarche.structureList;
    this.currentStep = StructuresListSteps.pmrAccess;
    // first filter screnn + regular StructureList form
    this.setStepNumber(Object.keys(StructuresListSteps).length);
    this.filtersForm = this.orientationUtils.createStructureOrientationForm(this.filtersForm.value.filters);
  }

  public setOnlineProceduresForm() {
    switch (this.onlineDemarchForm.get('accompanimentType').value) {
      case OnlineDemarche.structureList:
        this.currentType = OnlineDemarche.structureList;
        this.currentStep = StructuresListSteps.pmrAccess;
        this.setStepNumber(Object.keys(StructuresListSteps).length);
        this.onlineDemarchForm = this.orientationUtils.createStructureOrientationForm(
          this.utils.convertFiltersToModule(this.filters)
        );
        break;
      case OnlineDemarche.meetings:
        this.currentStep = OnlineDemarchesMeetingSteps.structureOrientator;
        this.currentType = OnlineDemarche.meetings;
        this.skipStructureOrientator();
        break;
      case OnlineDemarche.onlineMediation:
        this.currentType = OnlineDemarche.onlineMediation;
        this.currentStep = HotlineMediationSteps.mediationBeneciaryInfo;
        this.setStepNumber(Object.keys(HotlineMediationSteps).length);
        this.onlineDemarchForm = this.orientationUtils.createOnlineMediationForm(
          this.utils.convertFiltersToModule(this.filters)
        );
        break;
      default:
        throw new Error('Not implemented value in nextPage()');
    }
  }

  public skipStructureOrientator() {
    if (this.profile?.structuresLink?.length === 1) {
      // skip first step
      this.currentStep++;
    }
  }

  public setOnlineProcedureFilters(): void {
    // Handle online procedures
    const onlineProcedures: Filter[] = this.onlineDemarchForm.value.onlineDemarcheType.map((module: Module) => {
      return new Filter('onlineProcedures', module.id, module.displayText);
    });
    if (onlineProcedures.length > 0) this.filters.push(onlineProcedures[0]);
  }

  public setFilters(category: string): void {
    // Handle online procedures
    this.filters = this.filtersForm.value.filters.map((module: Module) => {
      return new Filter(category, module.id, module.displayText);
    });
  }

  public nextPage(): void {
    this.isPageValid = false;
    // Handle filters form
    if (
      this.isFilterForm() &&
      this.currentType === GenericOrientationSteps.common &&
      this.currentStep === FiltersSteps.filterChoice
    ) {
      switch (this.needType) {
        case NeedsType.equipmentAccess:
          this.setFilters('selfServiceMaterial');
          // Switch to structure list form
          this.setOnlineEquipmentsForm();
          break;
        case NeedsType.equipmentBuy:
          this.setFilters('solidarityMaterial');
          // Switch to structure list form
          this.setOnlineEquipmentsForm();
          break;
        case NeedsType.learnSkills:
          this.setFilters('baseSkills');
          this.needType = NeedsType.onlineDemarch;
          this.currentStep = OnlineDemarchesCommonSteps.accompanimentType;
          break;
      }
      return;
    }
    // Online demarches
    if (this.needType === NeedsType.onlineDemarch) {
      if (
        this.currentType === OnlineDemarche.common &&
        this.currentStep === OnlineDemarchesCommonSteps.accompanimentType
      ) {
        this.setOnlineProcedureFilters();
        this.setOnlineProceduresForm();
        return;
      }
      // Handle last screen Online appointment and print
      if (
        this.currentType === OnlineDemarche.onlineMediation &&
        this.currentStep === HotlineMediationSteps.mediationRecap
      ) {
        this.handleOnlineAppointment();
        return;
      }
      // Structure orientator screen
      if (
        this.currentType === OnlineDemarche.structureList &&
        this.currentStep === StructuresListSteps.structureOrientator
      ) {
        this.skipStructureOrientator();
        this.currentStep++;
        return;
      }
    }

    // Structure list handling
    if (this.isStructureListForm()) {
      // Print last screen
      if (this.currentStep === StructuresListSteps.mediationRecap) {
        this.printForm();
        return;
      }
      // Carto display case for full screen
      // Set to full screen
      if (this.currentStep === StructuresListSteps.address) {
        this.fullScreen = true;
      }
      // Unset fullscrenn
      if (this.currentStep === StructuresListSteps.structureChoice) {
        this.fullScreen = false;
      }
    }

    // Default case
    if (this.currentStep === null) {
      // handle the form start
      this.currentStep = 0;
      return;
    }
    if (this.currentStep < this.nbSteps) {
      this.currentStep++;
      this.checkLastStep();
    }
  }

  public prevPage(): void {
    // Structure list previous page
    if (this.isStructureListForm()) {
      if (this.currentStep === StructuresListSteps.structureChoice) {
        this.fullScreen = false;
        this.currentStep--;
        return;
      }
      if (this.currentStep === StructuresListSteps.structureOrientator) {
        this.fullScreen = true;
        this.currentStep--;
        return;
      }
    }

    // Default case
    if (this.currentStep === 0) {
      // handle the form start
      this.reset();
      return;
    }
    if (this.currentStep > 0) {
      this.currentStep--;
    }
  }

  public reset(): void {
    this.onlineDemarchForm = undefined;
    this.onlineDemarcheType = undefined;
    this.filtersForm = undefined;
    this.currentStep = null;
    this.currentType = OnlineDemarche.common;
    this.nbSteps = Object.keys(HotlineMediationSteps).length / 2;
    this.isPageValid = false;
    this.needType = undefined;
    this.filters = [];
  }

  /**
   * Return if it's a Structure List form
   * @returns
   */
  private isStructureListForm(): boolean {
    return (
      (this.needType === NeedsType.onlineDemarch ||
        this.needType === NeedsType.equipmentAccess ||
        this.needType === NeedsType.equipmentBuy) &&
      this.currentType === OnlineDemarche.structureList
    );
  }

  /**
   * Return if it's a filter form
   * @returns
   */
  private isFilterForm(): boolean {
    return (
      this.needType === NeedsType.equipmentAccess ||
      this.needType === NeedsType.equipmentBuy ||
      this.needType === NeedsType.learnSkills
    );
  }

  public canExit(): Promise<boolean> {
    // Avoid confirmation when user submit form and leave.
    if (this.canDeactivate || this.currentStep === null) {
      return new Promise((resolve) => resolve(true));
    } else {
      return new Promise((resolve) => this.showModal(resolve));
    }
  }
  private showModal(resolve: Function): void {
    this.showConfirmationModal = true;
    this.resolve = resolve;
  }
  public hasRedirectionAccepted(hasAccept: boolean): void {
    this.resolve(hasAccept);
    this.showConfirmationModal = false;
  }
}
